@@ -251,18 +251,25 @@ void explode_board( Board board ) {
   if ( ! board_overflow(board) ) {
     for( i = 0; i < board->bubbles->size; i++ ) {
       bubble = board->bubbles->element[i];
-      if ( bubble->state != RUNAWAY ) {
-	if ( bubble->state == STUCK )
-	  clock = ( 2 + bubble->cell/COLS - ROWS )*PROPAGATION_TIME;
-	else
-	  clock = 0;
+      if (( bubble->state != RUNAWAY )&&( bubble->state != STUCK )) {
+	bubble->vx = 0;
+	bubble->vy = 0;
+	set_bubble_state( bubble, EXPLODING, TOP_LAYER, 0 );
+      }
+    }
+    clock = 0;
+    for ( i = NB_CELLS-1; i >= 0; i-- )
+      if ( board->array->cell[i] != EMPTY_CELL ) {
+	bubble = board->array->cell[i];
 	bubble->vx = 0;
 	bubble->vy = 0;
 	set_bubble_state( bubble, EXPLODING, TOP_LAYER, clock );
+	clock -= PROPAGATION_TIME;
       }
-    }
     empty_cell_array( board->array );
   }
+  /* prevent from new row of bubbles */
+  board->launch_count = 0;
 }
 
 void drop_bubble( Board board, Bubble bubble ) {
@@ -285,6 +292,7 @@ void stick_bubble( Board board, Bubble bubble, int target_cell, int clock) {
 void animate_bubble( Board board, Bubble bubble, int dt ) {
   int i, processed = 0;
   double x, y, t;
+
   switch ( bubble->state ) {
   case NEW:
     if ( ! board->ready_to_fire ) {
@@ -310,7 +318,7 @@ void animate_bubble( Board board, Bubble bubble, int dt ) {
 
   case READY:
     update_countdown( board, dt );
-     if (( ! board->locked )&&( board->launch_requested )) {
+    if (( ! board->locked )&&( board->launch_requested )) {
       /* launch bubble */
       set_bubble_state( bubble, LAUNCHED, MEDIUM_LAYER, 0);
       bubble->vx = board->vx[board->canon_angle+CANON_ANGLE_MAX];
@@ -374,7 +382,13 @@ void animate_bubble( Board board, Bubble bubble, int dt ) {
       }
     if (( bubble->clock <= 0 )||( processed ))
       break;
-    
+
+    if ( bubble->clock > get_bubble_animation_duration(bubble) ) {
+      set_bubble_state( bubble, RUNAWAY, TOP_LAYER, 0);
+      remove_sprite_from_pool( board->sprite_pool, bubble->sprite );
+      break;
+    }
+
   case FALLING:
     bubble->y += dt*bubble->vy + 0.5*GRAVITY*dt*dt;
     bubble->vy += dt*GRAVITY;
@@ -510,12 +524,16 @@ void kill_board( Board board ) {
   Bubble bubble;
   for( i = 0; i < board->bubbles->size; i++ ) {
     bubble = board->bubbles->element[i];
-    if ( bubble->state == STUCK )
-      clock = -PROPAGATION_TIME*( ROWS - 1 - bubble->cell/COLS );
-    else
-      clock = 0;
-    set_bubble_state( bubble, DEAD, BOTTOM_LAYER, clock );
+    if (( bubble->state != RUNAWAY )&&( bubble->state != STUCK ))
+      set_bubble_state( bubble, DEAD, BOTTOM_LAYER, 0 );
   }
+  clock = 0;
+  for ( i = NB_CELLS-1; i >= 0; i-- )
+    if ( board->array->cell[i] != EMPTY_CELL ) {
+      bubble = board->array->cell[i];
+      set_bubble_state( bubble, DEAD, BOTTOM_LAYER, clock );
+      clock -= PROPAGATION_TIME;
+    }
   board->dead = 1;
 }
 
@@ -552,8 +570,8 @@ void move_board_canon( Board board, int dt ) {
   frame = (board->canon_angle+CANON_ANGLE_MAX)*canon_animation->size/NB_ANGLES;
   /* compute canon center coordinates */
   cframe = (Frame) canon_animation->element[frame];
-  x = scale_x( CANON_X, scale ) + cframe->width/2 - cframe->cx;
-  y = scale_y( CANON_Y, scale ) + cframe->height/2 - cframe->cy;
+  x = scale_x( CANON_X, scale );
+  y = scale_y( CANON_Y, scale );
   set_sprite_frame( board->canon_sprite, frame);
   set_sprite_position( board->canon_sprite, x, y);
   board->canon_direction = 0;
