@@ -46,7 +46,7 @@ extern int screen, depth;
 extern GC dialog_gc, dialog_text_gc;
 extern XFontStruct *dialog_font, *menu_font;
 extern Pixmap win_bg, board_bg;
-extern Pixmap diamond_on, diamond_off, diamond_on_mask, diamond_off_mask;
+extern Pixmap cup_on, cup_off, cup_on_mask, cup_off_mask;
 
 extern int scale, fps;
 extern unsigned long frame_duration;
@@ -69,7 +69,7 @@ struct _Game {
   Opponent opponent[2];
   GC board_gc;
   Pixmap board_pixmap[2];
-  Window board_window[2], msg_box[2], diamond_box[2], tags[2];
+  Window board_window[2], msg_box[2], cup_box[2], tags[2];
 };
 Game current_game = NULL;
 
@@ -84,7 +84,7 @@ char *player_name[][2] = {
   { PLAYER1_TAG, COMPUTER_TAG },
   { COMPUTER_TAG, COMPUTER_TAG }};
 
-void display_diamonds( Game game, int n, int on ) {
+void display_cups( Game game, int n, int on ) {
   
   Pixmap back;
   int i, x, y, width, height, pos = 0;  
@@ -96,25 +96,26 @@ void display_diamonds( Game game, int n, int on ) {
   gcv.function = GXcopy;
   gc = XCreateGC( display, root, gcm, &gcv);
   width = 2*scale;
-  height = board_spacing(scale);
+  height = 4*board_spacing(scale)/3;
+  
   x = game->board_x[n];
-  y = game->board_y[n] - 3*board_spacing(scale)/2;
+  y = game->board_y[n] - 5*board_spacing(scale)/3;
   back = XCreatePixmap( display, root, width, height, depth );
   XSetTSOrigin( display, dialog_gc, -x, -y );
   XFillRectangle( display, back, dialog_gc, 0, 0, width, height );
-  XSetClipMask( display, gc, diamond_off_mask );
+  XSetClipMask( display, gc, cup_off_mask );
   for ( i = 0; i < game->score[n]-1; i++ ) {
     XSetClipOrigin( display, gc, pos, 0 );
-    XCopyArea( display, diamond_off, back, gc, 0, 0, scale, height, pos, 0 );
+    XCopyArea( display, cup_off, back, gc, 0, 0, scale, height, pos, 0 );
     pos += scale;
   }
   if ( game->score[n] > 0 ) {
-    XSetClipMask( display, gc, (( on )? diamond_on_mask : diamond_off_mask ));
+    XSetClipMask( display, gc, (( on )? cup_on_mask : cup_off_mask ));
     XSetClipOrigin( display, gc, pos, 0 );
-    XCopyArea( display, (( on )? diamond_on : diamond_off ), back, gc,
+    XCopyArea( display, (( on )? cup_on : cup_off ), back, gc,
 	       0, 0, scale, height, pos, 0 );
   }
-  XSetWindowBackgroundPixmap( display, game->diamond_box[n], back );
+  XSetWindowBackgroundPixmap( display, game->cup_box[n], back );
   XFreePixmap( display, back );
   XFreeGC( display, gc );
 }
@@ -141,7 +142,7 @@ Game new_game( enum GameMode mode, int *handicap, int preload, int *colors,
   game->nb_boards = ( game->multi_player )? 2 : 1;
   /* create board(s) */
   for ( i = 0; i < game->nb_boards; i++ ) {
-    game->board[i] = new_board( PERIOD, handicap[i], 1-game->multi_player );
+    game->board[i] = new_board( PERIOD, handicap[i], ! preload );
     game->opponent[i] = new_opponent( game->board[i], level );
     if ( preload )
       load_bubbles( game->board[i], colors );
@@ -166,11 +167,11 @@ Game new_game( enum GameMode mode, int *handicap, int preload, int *colors,
   game->board_gc = XCreateGC( display, root, gcm, &gcv);
   /* create windows */
   for ( i = 0; i < game->nb_boards; i++ ) {
-    game->diamond_box[i] = 
+    game->cup_box[i] = 
       XCreateSimpleWindow( display, win, game->board_x[i],
-			   game->board_y[i] - 3*board_spacing(scale)/2, 
-			   2*scale, board_spacing(scale), 0, 0, 0 );
-    display_diamonds( game, i, False );
+			   game->board_y[i] - 5*board_spacing(scale)/3, 
+			   2*scale, 4*board_spacing(scale)/3, 0, 0, 0 );
+    display_cups( game, i, False );
     game->tags[i] = create_dialog( game->board_x[i] + board_win_width(scale)/2,
 				   game->board_y[i] - board_spacing(scale),
 				   player_name[mode][i], menu_font,
@@ -188,7 +189,7 @@ Game new_game( enum GameMode mode, int *handicap, int preload, int *colors,
     XSelectInput( display, game->board_window[i], ExposureMask );
     XSetWindowBackgroundPixmap( display, game->board_window[i], None );
     XMapWindow( display, game->board_window[i] );
-    XMapWindow( display, game->diamond_box[i] );
+    XMapWindow( display, game->cup_box[i] );
   }
   return game;
 }
@@ -204,7 +205,7 @@ void delete_game( Game game, int no_clean ) {
     delete_board( game->board[i] );
     XFreePixmap( display, game->board_pixmap[i] );
     XDestroyWindow( display, game->board_window[i] );
-    XDestroyWindow( display, game->diamond_box[i] );
+    XDestroyWindow( display, game->cup_box[i] );
     XDestroyWindow( display, game->tags[i] );
   }
   free(game);
@@ -496,13 +497,24 @@ static void timer_sleep( long ms ) {
     pause();
 }
 
-static void show_start_boxes( Game game ) {
-  static char msg[20];
+static void block_timer() {
   sigset_t set;
   sigemptyset( &set );
   sigaddset( &set, SIGALRM );
-  /* enter X critical section */
   sigprocmask( SIG_BLOCK, &set, NULL );
+}
+
+static void unblock_timer() {
+  sigset_t set;
+  sigemptyset( &set );
+  sigaddset( &set, SIGALRM );
+  sigprocmask( SIG_UNBLOCK, &set, NULL );
+}
+
+static void show_start_boxes( Game game ) {
+  static char msg[20];
+  /* enter X critical section */
+  block_timer();
   switch ( game->mode ) {
   case SINGLE_PLAYER:
     sprintf( msg, "Stage %d", game->round );
@@ -516,25 +528,19 @@ static void show_start_boxes( Game game ) {
     break;
   default:
     return;
-  } 
-  /* leave critical section */
-  sigprocmask( SIG_UNBLOCK, &set, NULL );
+  }
+  unblock_timer();
   /* sleep for 2 seconds */
   timer_sleep( 2000 );
-  /* enter X critical section */
-  sigprocmask( SIG_BLOCK, &set, NULL );
+  block_timer();
   hide_game_msg(game);
-  /* leave critical section */
-  sigprocmask( SIG_UNBLOCK, &set, NULL );
+  unblock_timer();
 }
 
 static void show_game_result( Game game ) {
   int i, on = 1, winner = -1;
-  sigset_t set;
-  sigemptyset( &set );
-  sigaddset( &set, SIGALRM );
   /* enter X critical section */
-  sigprocmask( SIG_BLOCK, &set, NULL );
+  block_timer();
   switch ( game->result ) {
   case PLAYER1_WON:
     show_board_msg( game, 0, "Win" );
@@ -559,24 +565,21 @@ static void show_game_result( Game game ) {
   case ABORTED:
     return;
   }
-  /* leave critical section */
-  sigprocmask( SIG_UNBLOCK, &set, NULL );
+  unblock_timer();
   /* sleep for 4 seconds */
   for ( i = 0; i < 20; i++ ) {
-    if ( winner >= 0 ) { 
-      sigprocmask( SIG_BLOCK, &set, NULL );
-      display_diamonds( game, winner, on );
-      XClearWindow( display, game->diamond_box[winner] );
-      sigprocmask( SIG_UNBLOCK, &set, NULL );
+    if ( winner >= 0 ) {
+      block_timer();
+      display_cups( game, winner, on );
+      XClearWindow( display, game->cup_box[winner] );
+      unblock_timer();
       on = 1 - on;
     }
     timer_sleep(200);
   }
-  /* enter X critical section */
-  sigprocmask( SIG_BLOCK, &set, NULL );
+  block_timer();
   hide_game_msg(game);
-  /* leave critical section */
-  sigprocmask( SIG_UNBLOCK, &set, NULL );
+  unblock_timer();
 }
 
 enum GameResult play_game( Game game ) {
